@@ -47,12 +47,7 @@ def main() -> None:
                 all_colors.append(opt.name)
     patch_color_list('kitty/rc/set_colors.py', nullable_colors, 'NULLABLE')
     patch_color_list('tools/cmd/at/set_colors.go', nullable_colors, 'NULLABLE')
-    patch_color_list('kittens/themes/collection.py', all_colors, 'ALL', ' ' * 8)
-
-    from kittens.diff.options.definition import definition as kd
-    write_output('kittens.diff', kd)
-    from kittens.ssh.options.definition import definition as sd
-    write_output('kittens.ssh', sd)
+    patch_color_list('tools/themes/collection.go', all_colors, 'ALL')
 
 
 if __name__ == '__main__':
