@@ -201,6 +201,11 @@ historybuf_mark_line_dirty(HistoryBuf *self, index_type y) {
     attrptr(self, index_of(self, y))->has_dirty_text = true;
 }
 
+void
+historybuf_set_line_has_image_placeholders(HistoryBuf *self, index_type y, bool val) {
+    attrptr(self, index_of(self, y))->has_image_placeholders = val;
+}
+
 void
 historybuf_clear(HistoryBuf *self) {
     pagerhist_clear(self);
