@@ -7,12 +7,20 @@ import tempfile
 
 from kitty.config import build_ansi_color_table, defaults
 from kitty.fast_data_types import (
-    Color, ColorProfile, Cursor as C, HistoryBuf, LineBuf,
-    parse_input_from_terminal, strip_csi, truncate_point_for_length, wcswidth,
-    wcwidth
+    Color,
+    ColorProfile,
+    HistoryBuf,
+    LineBuf,
+    expand_ansi_c_escapes,
+    parse_input_from_terminal,
+    strip_csi,
+    truncate_point_for_length,
+    wcswidth,
+    wcwidth,
 )
+from kitty.fast_data_types import Cursor as C
 from kitty.rgb import to_color
-from kitty.utils import is_path_in_temp_dir, sanitize_title
+from kitty.utils import is_path_in_temp_dir, sanitize_title, sanitize_url_for_dispay_to_user
 
 from . import BaseTest, filled_cursor, filled_history_buf, filled_line_buf
 
@@ -20,11 +28,10 @@ from . import BaseTest, filled_cursor, filled_history_buf, filled_line_buf
 def create_lbuf(*lines):
     maxw = max(map(len, lines))
     ans = LineBuf(len(lines), maxw)
-    prev_full_length = False
     for i, l0 in enumerate(lines):
         ans.line(i).set_text(l0, 0, len(l0), C())
-        ans.set_continued(i, prev_full_length)
-        prev_full_length = len(l0) == maxw
+        if i > 0:
+            ans.set_continued(i, len(lines[i-1]) == maxw)
     return ans
 
 
@@ -73,9 +80,9 @@ class TestDataTypes(BaseTest):
                 self.assertFalse(c.reverse)
                 self.assertTrue(c.bold)
         self.assertFalse(old.is_continued(0))
-        old.set_continued(0, True)
-        self.assertTrue(old.is_continued(0))
-        self.assertFalse(old.is_continued(1))
+        old.set_continued(1, True)
+        self.assertTrue(old.is_continued(1))
+        self.assertFalse(old.is_continued(0))
 
         lb = filled_line_buf(5, 5, filled_cursor())
         lb2 = LineBuf(5, 5)
@@ -398,9 +405,10 @@ class TestDataTypes(BaseTest):
         self.ae(tpl('a\U0001f337', 2), 1)
         self.ae(tpl('a\U0001f337', 3), 2)
         self.ae(tpl('a\U0001f337b', 4), 3)
-        self.ae(sanitize_title('a\0\01 \t\n\f\rb'), 'a b')
         self.ae(tpl('a\x1b[31mbc', 2), 7)
 
+        self.ae(sanitize_title('a\0\01 \t\n\f\rb'), 'a b')
+
         def tp(*data, leftover='', text='', csi='', apc='', ibp=False):
             text_r, csi_r, apc_r, rest = [], [], [], []
             left = ''
@@ -436,13 +444,15 @@ class TestDataTypes(BaseTest):
                 self.assertTrue(is_path_in_temp_dir(os.path.join(prefix, path)))
         for path in ('/home/xy/d.png', '/tmp/../home/x.jpg'):
             self.assertFalse(is_path_in_temp_dir(os.path.join(path)))
+        self.ae(sanitize_url_for_dispay_to_user(
+            'h://a\u0430b.com/El%20Ni%C3%B1o/'), 'h://xn--ab-7kc.com/El Niño/')
 
     def test_color_profile(self):
         c = ColorProfile()
         c.update_ansi_color_table(build_ansi_color_table())
         for i in range(8):
             col = getattr(defaults, f'color{i}')
-            self.assertEqual(c.as_color(i << 8 | 1), col)
+            self.ae(c.as_color(i << 8 | 1), col)
         self.ae(c.as_color(255 << 8 | 1), Color(0xee, 0xee, 0xee))
 
     def test_historybuf(self):
@@ -517,7 +527,7 @@ class TestDataTypes(BaseTest):
         self.ae(l2.as_ansi(), '\x1b[1;2;3;7;9;34;48:2:1:2:3;58:5:5m' '1'
                 '\x1b[22;23;27;29;39;49;59m' '0000')
         lb = filled_line_buf()
-        for i in range(lb.ynum):
+        for i in range(1, lb.ynum + 1):
             lb.set_continued(i, True)
         a = []
         lb.as_ansi(a.append)
@@ -536,9 +546,7 @@ class TestDataTypes(BaseTest):
         q('a\x1b[12;34:43mbc', 'abc')
 
     def test_single_key(self):
-        from kitty.fast_data_types import (
-            GLFW_MOD_KITTY, GLFW_MOD_SHIFT, SingleKey
-        )
+        from kitty.fast_data_types import GLFW_MOD_KITTY, GLFW_MOD_SHIFT, SingleKey
         for m in (GLFW_MOD_KITTY, GLFW_MOD_SHIFT):
             s = SingleKey(mods=m)
             self.ae(s.mods, m)
@@ -567,3 +575,25 @@ class TestDataTypes(BaseTest):
             self.assertNotIn(b'\x1b[201~', q)
             self.assertNotIn('\x9b201~'.encode('utf-8'), q)
             self.assertIn(b'ab', q)
+
+    def test_expand_ansi_c_escapes(self):
+        for src, expected in {
+            'abc': 'abc',
+            r'a\ab': 'a\ab',
+            r'a\eb': 'a\x1bb',
+            r'a\r\nb': 'a\r\nb',
+            r'a\c b': 'a\0b',
+            r'a\c': 'a\\c',
+            r'a\x1bb': 'a\x1bb',
+            r'a\x1b': 'a\x1b',
+            r'a\x1': 'a\x01',
+            r'a\x1g': 'a\x01g',
+            r'a\z\"': 'a\\z"',
+            r'a\123b': 'a\123b',
+            r'a\128b': 'a\0128b',
+            r'a\u1234e': 'a\u1234e',
+            r'a\U1f1eez': 'a\U0001f1eez',
+            r'a\x1\\':    "a\x01\\",
+        }.items():
+            actual = expand_ansi_c_escapes(src)
+            self.ae(expected, actual)
